@@ -62,7 +62,7 @@ browser.tabs.onActivated.addListener(async function(activeInfo) {
             }
         }
     } catch (err) {
-        console.log('Error: ' + err);
+        console.log('Error: ' + err.message);
     }
 });
 
@@ -98,6 +98,11 @@ browser.webNavigation.onCommitted.addListener((details) => {
         return;
     }
 
+    // Clear credentials on reload so a new retrieval can be made
+    if (details.transitionType === 'reload') {
+        page.clearLogins(details.tabId);
+    }
+
     page.redirectCount = 0;
 });
 
@@ -109,7 +114,8 @@ const contextMenuItems = [
     { title: tr('contextMenuFillTOTP'), action: 'fill_totp' },
     { title: tr('contextMenuFillAttribute'), id: 'fill_attribute', visible: false },
     { title: tr('contextMenuShowPasswordGenerator'), action: 'show_password_generator' },
-    { title: tr('contextMenuSaveCredentials'), action: 'remember_credentials' }
+    { title: tr('contextMenuSaveCredentials'), action: 'save_credentials' },
+    { title: tr('contextMenuRequestGlobalAutoType'), action: 'request_autotype' }
 ];
 
 const menuContexts = [ 'editable' ];
@@ -137,7 +143,11 @@ for (const item of contextMenuItems) {
 
 // Listen for keyboard shortcuts specified by user
 browser.commands.onCommand.addListener(async (command) => {
-    if (contextMenuItems.some(e => e.action === command)) {
+    if (contextMenuItems.some(e => e.action === command)
+        || command === 'redetect_fields'
+        || command === 'choose_credential_fields'
+        || command === 'retrive_credentials_forced'
+        || command === 'reload_extension') {
         const tabs = await browser.tabs.query({ active: true, currentWindow: true });
         if (tabs.length) {
             browser.tabs.sendMessage(tabs[0].id, { action: command });
