@@ -1,10 +1,24 @@
 'use strict';
 
+// Site Preferences ignore options
 const IGNORE_NOTHING = 'ignoreNothing';
 const IGNORE_NORMAL = 'ignoreNormal';
+const IGNORE_AUTOSUBMIT = 'ignoreAutoSubmit';
 const IGNORE_FULL = 'ignoreFull';
 
-const schemeSegment = '(\\*|http|https|ws|wss|file|ftp)';
+// Credential sorting options
+const SORT_BY_TITLE = 'sortByTitle';
+const SORT_BY_USERNAME = 'sortByUsername';
+const SORT_BY_GROUP_AND_TITLE = 'sortByGroupAndTitle';
+const SORT_BY_GROUP_AND_USERNAME = 'sortByGroupAndUsername';
+
+// Update check intervals
+const CHECK_UPDATE_NEVER = 0;
+const CHECK_UPDATE_THREE_DAYS = 3;
+const CHECK_UPDATE_ONE_WEEK = 7;
+const CHECK_UPDATE_ONE_MONTH = 30;
+
+const schemeSegment = '(\\*|http|https|ws|wss|ftp)';
 const hostSegment = '(\\*|(?:\\*\\.)?(?:[^/*]+))?';
 const pathSegment = '(.*)';
 
@@ -53,6 +67,19 @@ const matchPatternToRegExp = function(pattern) {
         return (/^(?:http|https|file|ftp|app):\/\//);
     }
 
+    // special handling of file:// since there is no host
+    if (pattern.startsWith('file://')) {
+        let regex = '^';
+        pattern = pattern.replace(/\./g, '\\.');
+        if (pattern.endsWith('*')) {
+            regex += pattern.slice(0, -1);
+        }
+        else {
+            regex += `${pattern}$`;
+        }
+        return new RegExp(regex);
+    }
+
     const matchPatternRegExp = new RegExp(
         `^${schemeSegment}://${hostSegment}/${pathSegment}$`
     );
@@ -113,6 +140,19 @@ const slashNeededForUrl = function(pattern) {
     return matchPattern.exec(pattern);
 };
 
+// Returns the top level domain, e.g. https://another.example.co.uk -> example.co.uk
+// This is done because a top level domain will probably give better matches with Auto-Type than a full hostname.
+const getTopLevelDomainFromUrl = function(hostname) {
+    const domainRegex = new RegExp(/(\w+).(com|net|org|edu|co)*(.\w+)$/g);
+    const domainMatch = domainRegex.exec(hostname);
+
+    if (domainMatch) {
+        return domainMatch[0];
+    }
+
+    return hostname;
+};
+
 function tr(key, params) {
     return browser.i18n.getMessage(key, params);
 }
