@@ -6,7 +6,6 @@ const _maximumMutations = 200;
 // Contains already called method names
 const _called = {};
 _called.automaticRedetectCompleted = false;
-_called.clearLogins = false;
 _called.retrieveCredentials = false;
 
 // Wrapper
@@ -51,7 +50,7 @@ kpxcIcons.addIcon = async function(field, iconType) {
 // Adds all necessary icons to a saved form
 kpxcIcons.addIconsFromForm = async function(form) {
     const addUsernameIcons = async function(c) {
-        if (kpxc.settings.showLoginFormIcon && await kpxc.passwordFilled() === false) {
+        if (kpxc.settings.showLoginFormIcon && await kpxc.passwordFilledWithExceptions(c) === false) {
             // Special case where everything else has been hidden, but a single password field is now displayed.
             // For example PayPal and Amazon is handled like this.
             if (c.username && !c.password && c.passwordInputs.length === 1) {
@@ -155,22 +154,16 @@ kpxcForm.getCredentialFieldsFromForm = function(form) {
 
 // Get the form submit button instead if action URL is same as the page itself
 kpxcForm.getFormSubmitButton = function(form) {
-    if (!form.action || typeof form.action !== 'string') {
+    if (!form || !form.action || typeof form.action !== 'string') {
         return;
     }
 
     const action = kpxc.submitUrl || form.action;
 
-    // Special handling for accounts.google.com. The submit button is outside the form.
-    if (form.action.startsWith(kpxcSites.googleUrl)) {
-        const findDiv = $('#identifierNext, #passwordNext');
-        if (!findDiv) {
-            return undefined;
-        }
-
-        const buttons = findDiv.getElementsByTagName('button');
-        kpxcSites.savedForm = form;
-        return buttons.length > 0 ? buttons[0] : undefined;
+    // Check if the site needs a special handling for retrieving the form submit button
+    const exceptionButton = kpxcSites.formSubmitButtonExceptionFound(form);
+    if (exceptionButton) {
+        return exceptionButton;
     }
 
     if (action.includes(document.location.origin + document.location.pathname)) {
@@ -182,8 +175,8 @@ kpxcForm.getFormSubmitButton = function(form) {
     }
 
     // Try to find another button. Select the last one.
-    // TODO: Possibly change this behavior to select the last one for only certain sites.
-    const buttons = Array.from(form.querySelectorAll(kpxcForm.formButtonQuery));
+    // If any formaction overriding the default action is set, ignore those buttons.
+    const buttons = Array.from(form.querySelectorAll(kpxcForm.formButtonQuery)).filter(b => !b.getAttribute('formAction'));
     if (buttons.length > 0) {
         return buttons[buttons.length - 1];
     }
@@ -261,6 +254,11 @@ kpxcForm.onSubmit = async function(e) {
         form = kpxcSites.savedForm;
     }
 
+    // Still not found? Try using the first one from kpxcForm.savedForms
+    if (!form && kpxcForm.savedForms.length > 0) {
+        form = kpxcForm.savedForms[0].form;
+    }
+
     if (!form) {
         return;
     }
@@ -289,7 +287,9 @@ kpxcForm.onSubmit = async function(e) {
         return;
     }
 
-    await kpxc.setPasswordFilled(true);
+    if (passwordField) {
+        await kpxc.setPasswordFilled(true);
+    }
 
     const url = trimURL(kpxc.settings.saveDomainOnlyNewCreds ? window.top.location.origin : window.top.location.href);
     await sendMessage('page_set_submitted', [ true, usernameValue, passwordValue, url, kpxc.credentials ]);
@@ -305,6 +305,7 @@ kpxcForm.saveForm = function(form, combination) {
         username: combination.username,
         password: combination.password,
         totp: combination.totp,
+        totpInputs: Array.from(form.elements).filter(e => e.nodeName === 'INPUT' && kpxcTOTPIcons.isValid(e)),
         passwordInputs: Array.from(form.elements).filter(e => e.nodeName === 'INPUT' && e.type === 'password')
     });
 };
@@ -364,6 +365,53 @@ kpxcFields.getAllCombinations = async function(inputs) {
         combinations.push(combination);
     }
 
+    // Check for multiple segmented TOTP fields
+    if (combinations.length === 0) {
+        kpxcFields.getSegmentedTOTPFields(inputs, combinations);
+    }
+
+    return combinations;
+};
+
+// Adds segmented TOTP fields to the combination if found
+kpxcFields.getSegmentedTOTPFields = function(inputs, combinations) {
+    const addTotpFieldsToCombination = function(inputFields) {
+        const totpInputs = Array.from(inputFields).filter(e => e.nodeName === 'INPUT' && e.type !== 'password');
+        if (totpInputs.length === 6) {
+            const combination = {
+                form: form,
+                totpInputs: totpInputs,
+                username: null,
+                password: null,
+                passwordInputs: []
+            };
+
+            combinations.push(combination);
+
+            // Create an icon to the right side of the segmented fields
+            kpxcTOTPIcons.newIcon(totpInputs[totpInputs.length - 1], kpxc.databaseState, true);
+            kpxcIcons.icons.push({
+                field: totpInputs[totpInputs.length - 1],
+                iconType: kpxcIcons.iconTypes.TOTP,
+                segmented: true
+            });
+        }
+    };
+
+    const form = inputs.length > 0 ? inputs[0].form : undefined;
+    if (form && (acceptedOTPFields.some(f => (form.className && form.className.includes(f))
+        || (form.id && typeof(form.id) === 'string' && form.id.includes(f))
+        || (form.name && typeof(form.name) === 'string' && form.name.includes(f))
+        || form.length === 6))) {
+        // Use the form's elements
+        addTotpFieldsToCombination(form.elements);
+    } else if (inputs.length === 6 && inputs.every(i => (i.inputMode === 'numeric' && i.pattern.includes('0-9'))
+                || (i.type === 'text' && i.maxLength == 1)
+                || i.type === 'tel')) {
+        // No form is found, but input fields are possibly segmented TOTP fields
+        addTotpFieldsToCombination(inputs);
+    }
+
     return combinations;
 };
 
@@ -408,8 +456,8 @@ kpxcFields.getCombination = async function(field, givenType) {
     for (const combination of kpxc.combinations) {
         if (!givenType && Object.values(combination).find(c => c === field)) {
             return combination;
-        } else if (givenType) {
-            if (combination[givenType] === field) {
+        } else if (givenType && combination[givenType]) {
+            if (combination[givenType] === field || combination[givenType].includes(field)) {
                 return combination;
             }
         }
@@ -418,8 +466,75 @@ kpxcFields.getCombination = async function(field, givenType) {
     return undefined;
 };
 
-// Gets of generates an unique ID for the element
-kpxcFields.getId = function(target) {
+// Sets and returns unique ID's for the element
+kpxcFields.setId = function(target) {
+    return [ kpxcFields.getIdFromXPath(target), kpxcFields.getIdFromProperties(target) ];
+};
+
+// Returns generated unique ID's for the element. If XPath ID fails, return the fallback one.
+kpxcFields.getId = function(idArray, inputField) {
+    if (!idArray) {
+        return '';
+    }
+
+    // Legacy ID is used. Convert it to the new one if possible
+    if (!Array.isArray(idArray) && idArray.length > 0) {
+        if (idArray === kpxcFields.getLegacyId(inputField)) {
+            idArray = kpxcFields.setId(inputField);
+        }
+    }
+
+    const elementFromXPath = kpxcFields.getElementFromXPathId(idArray[0]);
+    const fallbackId = kpxcFields.getIdFromProperties(inputField);
+
+    return elementFromXPath || (fallbackId === idArray[1] ? inputField : '');
+};
+
+// Returns element XPath
+kpxcFields.getIdFromXPath = function(target) {
+    let xpath = '';
+    let pos;
+    let temp;
+
+    while (target !== document.documentElement) {
+        pos = 0;
+        temp = target;
+        while (temp) {
+            if (temp.nodeType === 1 && temp.nodeName === target.nodeName) {
+                pos += 1;
+            }
+
+            temp = temp.previousSibling;
+        }
+
+        xpath = `${target.nodeName.toLowerCase()}${(pos > 1 ? `[${pos}]/` : '/')}${xpath}`;
+        target = target.parentNode;
+    }
+
+    xpath = `/${document.documentElement.nodeName.toLowerCase()}/${xpath}`;
+    xpath = xpath.replace(/\/$/, '');
+    return xpath;
+};
+
+// Generate uniqe ID from properties (new method)
+kpxcFields.getIdFromProperties = function(target) {
+    if (target.name) {
+        return `${target.nodeName} ${target.type} ${target.name} ${target.placeholder}`;
+    }
+
+    if (target.classList && target.classList.length > 0) {
+        return `${target.nodeName} ${target.type} ${target.classList.value} ${target.placeholder}`;
+    }
+
+    if (target.id && target.id !== '') {
+        return `${target.nodeName} ${target.type} ${kpxcFields.prepareId(target.id)} ${target.placeholder}`;
+    }
+
+    return `kpxc ${target.type} ${target.clientTop}${target.clientLeft}${target.clientWidth}${target.clientHeight}${target.offsetTop}${target.offsetLeft}`;
+};
+
+// Legacy unique ID generation for converting
+kpxcFields.getLegacyId = function(target) {
     if (target.classList.length > 0) {
         return `${target.nodeName} ${target.type} ${target.classList.value} ${target.name} ${target.placeholder}`;
     }
@@ -431,6 +546,10 @@ kpxcFields.getId = function(target) {
     return `kpxc ${target.type} ${target.clientTop}${target.clientLeft}${target.clientWidth}${target.clientHeight}${target.offsetTop}${target.offsetLeft}`;
 };
 
+kpxcFields.getElementFromXPathId = function(xpath) {
+    return (new XPathEvaluator()).evaluate(xpath, document.documentElement, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
+};
+
 // Check for new password via autocomplete attribute
 kpxcFields.isAutocompleteAppropriate = function(field) {
     const autocomplete = field.getLowerCaseAttribute('autocomplete');
@@ -440,7 +559,7 @@ kpxcFields.isAutocompleteAppropriate = function(field) {
 // Checks if Custom Login Fields are used for the site
 kpxcFields.isCustomLoginFieldsUsed = function() {
     const location = kpxc.getDocumentLocation();
-    return kpxc.settings['defined-custom-fields'] && kpxc.settings['defined-custom-fields'][location];
+    return kpxc.settings['defined-custom-fields'] !== undefined && kpxc.settings['defined-custom-fields'][location] !== undefined;
 };
 
 // Returns true if form is a search form
@@ -500,7 +619,9 @@ kpxcFields.isVisible = function(elem) {
 
     // Check CSS visibility
     const elemStyle = getComputedStyle(elem);
+    const opacity = Number(elemStyle.opacity);
     if (elemStyle.visibility && (elemStyle.visibility === 'hidden' || elemStyle.visibility === 'collapse')
+        || (opacity < MIN_OPACITY || opacity > MAX_OPACITY)
         || parseInt(elemStyle.width, 10) <= MIN_INPUT_FIELD_WIDTH_PX
         || parseInt(elemStyle.height, 10) <= MIN_INPUT_FIELD_WIDTH_PX) {
         return false;
@@ -544,9 +665,9 @@ kpxcFields.useCustomLoginFields = async function() {
     }
 
     // Finds the input field based on the stored ID
-    const findInputField = async function(inputFields, id) {
-        if (id) {
-            const input = inputFields.find(e => kpxcFields.getId(e) === id);
+    const findInputField = async function(inputFields, idArray) {
+        if (idArray) {
+            const input = inputFields.find(e => e === kpxcFields.getId(idArray, e));
             if (input) {
                 return input;
             }
@@ -581,7 +702,13 @@ kpxcFields.useCustomLoginFields = async function() {
     // Handle custom TOTP field
     if (totp) {
         totp.setAttribute('kpxc-defined', 'totp');
-        kpxcTOTPIcons.newIcon(totp, kpxc.databaseState, true);
+        kpxcTOTPIcons.newIcon(totp, kpxc.databaseState);
+    }
+
+    // If not all expected fields are identified, return an empty combination
+    if ((creds.username && !username) || (creds.password && !password) || (creds.totp && !totp)
+        || (creds.fields.length !== stringFields.length)) {
+        return [];
     }
 
     const combinations = [];
@@ -632,7 +759,7 @@ kpxc.url = null;
 // Add page to Site Preferences with Username-only detection enabled. Set from the popup
 kpxc.addToSitePreferences = async function() {
     // Returns a predefined URL for certain sites
-    let site = trimURL(window.top.location.href);
+    let site = trimURL(window.top.location.href).toLowerCase();
 
     // Check if the site already exists -> update the current settings
     let siteExists = false;
@@ -663,11 +790,11 @@ kpxc.addToSitePreferences = async function() {
 kpxc.clearAllFromPage = function() {
     kpxc.credentials = [];
     kpxc.inputs = [];
-    kpxcAutocomplete.elements = [];
+    kpxcUserAutocomplete.clear();
     _called.retrieveCredentials = false;
 
     if (kpxc.settings.autoCompleteUsernames) {
-        kpxcAutocomplete.closeList();
+        kpxcUserAutocomplete.closeList();
     }
 
     // Switch back to default popup
@@ -756,15 +883,16 @@ kpxc.fillInFromActiveElement = async function(passOnly = false) {
             return;
         }
 
-        // set focus to the input field
+        // Set focus to the input field
         field.focus();
 
         if (kpxc.credentials.length > 1) {
             // More than one credential -> show autocomplete list
-            kpxcAutocomplete.showList(field);
-            return
+            kpxcUserAutocomplete.showList(field);
+            return;
         } else {
             // Just one credential -> fill the first combination found
+            await sendMessage('page_set_login_id', kpxc.credentials[0].uuid);
             kpxc.fillInCredentials(combination, kpxc.credentials[0].login, kpxc.credentials[0].uuid, passOnly);
             return;
         }
@@ -787,7 +915,7 @@ kpxc.fillInFromActiveElement = async function(passOnly = false) {
         return;
     }
 
-    await sendMessage('page_set_login_id', 0);
+    await sendMessage('page_set_login_id', kpxc.credentials[0].uuid);
     kpxc.fillInCredentials(combination, kpxc.credentials[0].login, kpxc.credentials[0].uuid, passOnly);
 };
 
@@ -798,7 +926,7 @@ kpxc.fillFromAutofill = async function() {
     }
 
     const index = kpxc.combinations.length - 1;
-    await sendMessage('page_set_login_id', 0);
+    await sendMessage('page_set_login_id', kpxc.credentials[0].uuid);
     kpxc.fillInCredentials(kpxc.combinations[index], kpxc.credentials[0].login, kpxc.credentials[0].uuid);
 
     // Generate popup-list of usernames + descriptions
@@ -811,55 +939,104 @@ kpxc.fillFromPopup = async function(id, uuid) {
         return;
     }
 
-    await sendMessage('page_set_login_id', id);
-    kpxc.fillInCredentials(kpxc.combinations[0], kpxc.credentials[id].login, uuid);
-    kpxcAutocomplete.closeList();
+    await sendMessage('page_set_login_id', uuid);
+    const selectedCredentials = kpxc.credentials.find(c => c.uuid === uuid);
+    if (!selectedCredentials) {
+        console.log('Error: Uuid not found: ', uuid);
+        return;
+    }
+
+    kpxc.fillInCredentials(kpxc.combinations[0], selectedCredentials.login, uuid);
+    kpxcUserAutocomplete.closeList();
 };
 
 // Fill requested from TOTP icon
 kpxc.fillFromTOTP = async function(target) {
     const el = target || document.activeElement;
-    let index = await sendMessage('page_get_login_id');
+    const credentialList = await kpxc.updateTOTPList();
 
-    // Use the first credential available if not set
-    if (index === undefined) {
-        index = 0;
+    if (credentialList && credentialList.length === 0) {
+        kpxcUI.createNotification('warning', tr('credentialsNoTOTPFound'));
+        return;
     }
 
-    if (index >= 0 && kpxc.credentials[index]) {
-        // Check the value from StringFields
-        if (kpxc.credentials[index].totp && kpxc.credentials[index].totp.length > 0) {
-            // Retrieve a new TOTP value
-            const totp = await sendMessage('get_totp', [ kpxc.credentials[index].uuid, kpxc.credentials[index].totp ]);
-            if (!totp) {
-                kpxcUI.createNotification('warning', tr('credentialsNoTOTPFound'));
-                return;
-            }
+    if (credentialList && credentialList.length === 1) {
+        kpxc.fillTOTPFromUuid(el, credentialList[0].uuid);
+        return;
+    }
 
-            kpxc.setValue(el, totp);
-        } else if (kpxc.credentials[index].stringFields && kpxc.credentials[index].stringFields.length > 0) {
-            const stringFields = kpxc.credentials[index].stringFields;
-            for (const s of stringFields) {
-                const val = s['KPH: {TOTP}'];
-                if (val) {
-                    kpxc.setValue(el, val);
-                }
+    kpxcTOTPAutocomplete.showList(el, true);
+};
+
+// Fill TOTP with matching uuid
+kpxc.fillTOTPFromUuid = async function(el, uuid) {
+    if (!el || !uuid) {
+        return;
+    }
+
+    const user = kpxc.credentials.find(c => c.uuid === uuid);
+    if (!user) {
+        return;
+    }
+
+    if (user.totp && user.totp.length > 0) {
+        // Retrieve a new TOTP value
+        const totp = await sendMessage('get_totp', [ user.uuid, user.totp ]);
+        if (!totp) {
+            kpxcUI.createNotification('warning', tr('credentialsNoTOTPFound'));
+            return;
+        }
+
+        kpxc.setTOTPValue(el, totp);
+    } else if (user.stringFields && user.stringFields.length > 0) {
+        const stringFields = user.stringFields;
+        for (const s of stringFields) {
+            const val = s['KPH: {TOTP}'];
+            if (val) {
+                kpxc.setTOTPValue(el, val);
             }
         }
     }
 };
 
+// Set normal or segmented TOTP value
+kpxc.setTOTPValue = function(elem, val) {
+    if (kpxc.combinations.length === 0) {
+        return;
+    }
+
+    for (const comb of kpxc.combinations) {
+        if (comb.totpInputs && comb.totpInputs.length === 6) {
+            kpxc.fillSegmentedTotp(elem, val, comb.totpInputs);
+            return;
+        }
+    }
+
+    kpxc.setValue(elem, val);
+};
+
+// Fill TOTP in parts
+kpxc.fillSegmentedTotp = function(elem, val, totpInputs) {
+    if (!totpInputs.includes(elem)) {
+        return;
+    }
+
+    for (let i = 0; i < 6; ++i) {
+        kpxc.setValue(totpInputs[i], val[i]);
+    }
+};
+
 // Fill requested from username icon
 kpxc.fillFromUsernameIcon = async function(combination) {
     await kpxc.receiveCredentialsIfNecessary();
     if (kpxc.credentials.length === 0) {
         return;
     } else if (kpxc.credentials.length > 1 && kpxc.settings.autoCompleteUsernames) {
-        kpxcAutocomplete.showList(combination.username || combination.password);
+        kpxcUserAutocomplete.showList(combination.username || combination.password);
         return;
     }
 
-    await sendMessage('page_set_login_id', 0);
+    await sendMessage('page_set_login_id', kpxc.credentials[0].uuid);
     kpxc.fillInCredentials(combination, kpxc.credentials[0].login, kpxc.credentials[0].uuid);
 };
 
@@ -890,6 +1067,7 @@ kpxc.fillInCredentials = async function(combination, predefinedUsername, uuid, p
     // Find the correct credentials
     const selectedCredentials = kpxc.credentials.find(c => c.uuid === uuid);
     if (!selectedCredentials) {
+        console.log('Error: Uuid not found: ', uuid);
         return;
     }
 
@@ -913,22 +1091,23 @@ kpxc.fillInCredentials = async function(combination, predefinedUsername, uuid, p
     }
 
     // Fill StringFields
-    if (selectedCredentials.stringFields.length > 0) {
+    if (selectedCredentials.stringFields && selectedCredentials.stringFields.length > 0) {
         kpxc.fillInStringFields(combination.fields, selectedCredentials.stringFields);
     }
 
     // Close autocomplete menu after fill
-    kpxcAutocomplete.closeList();
+    kpxcUserAutocomplete.closeList();
 
     // Reset ManualFill
     await sendMessage('page_set_manual_fill', ManualFill.NONE);
 
     // Auto-submit
-    if (kpxc.settings.autoSubmit && !skipAutoSubmit) {
+    const autoSubmitIgnoredForSite = await kpxc.siteIgnored(IGNORE_AUTOSUBMIT);
+    if (kpxc.settings.autoSubmit && !skipAutoSubmit && !autoSubmitIgnoredForSite) {
         const submitButton = kpxcForm.getFormSubmitButton(combination.form);
         if (submitButton !== undefined) {
             submitButton.click();
-        } else {
+        } else if (combination.form) {
             combination.form.submit();
         }
     }
@@ -982,7 +1161,9 @@ kpxc.getFormActionUrl = function(combination) {
     }
 
     if (typeof(action) !== 'string' || action === '') {
-        action = document.location.origin + document.location.pathname;
+        // Firefox can report location.origin as 'null' with localHost files
+        const origin = document.location.origin === 'null' ? 'file://' : document.location.origin;
+        action = origin + document.location.pathname;
     }
 
     return action;
@@ -1064,10 +1245,14 @@ kpxc.initAutocomplete = function() {
 
     for (const c of kpxc.combinations) {
         if (c.username) {
-            kpxcAutocomplete.create(c.username, false, kpxc.settings.autoSubmit);
+            kpxcUserAutocomplete.create(c.username, false, kpxc.settings.autoSubmit);
         } else if (!c.username && c.password) {
             // Single password field
-            kpxcAutocomplete.create(c.password, false, kpxc.settings.autoSubmit);
+            kpxcUserAutocomplete.create(c.password, false, kpxc.settings.autoSubmit);
+        }
+
+        if (c.totp) {
+            kpxcTOTPAutocomplete.create(c.totp, false, kpxc.settings.autoSubmit);
         }
     }
 };
@@ -1078,10 +1263,15 @@ kpxc.initCombinations = async function(inputs = []) {
         return [];
     }
 
-    const combinations = kpxcFields.isCustomLoginFieldsUsed()
+    const isCustomLoginFieldsUsed = kpxcFields.isCustomLoginFieldsUsed();
+    const combinations = isCustomLoginFieldsUsed
                        ? await kpxcFields.useCustomLoginFields()
                        : await kpxcFields.getAllCombinations(inputs);
     if (!combinations || combinations.length === 0) {
+        if (isCustomLoginFieldsUsed) {
+            kpxcUI.createNotification('warning', tr('optionsCustomFieldsNotFound'));
+        }
+
         return [];
     }
 
@@ -1104,9 +1294,6 @@ kpxc.initCombinations = async function(inputs = []) {
 
 // The main function for finding input fields
 kpxc.initCredentialFields = async function() {
-    await sendMessage('page_clear_logins', _called.clearLogins);
-    _called.clearLogins = true;
-
     // Identify all forms in the page
     const formInputs = await kpxc.identifyFormInputs();
 
@@ -1144,25 +1331,46 @@ kpxc.initCredentialFields = async function() {
     }
 };
 
-// Intializes the login popup list for choosing credentials
+// Intializes the login lists for popup and Autocomplete Menu
 kpxc.initLoginPopup = function() {
     if (kpxc.credentials.length === 0) {
         return;
     }
 
-    const getLoginText = function(credential, withGroup) {
-        const name = credential.name.length < MAX_AUTOCOMPLETE_NAME_LEN
+    // Returns a login item with additional information for sorting
+    const getLoginItem = function(credential, withGroup, loginId) {
+        const title = credential.name.length < MAX_AUTOCOMPLETE_NAME_LEN
                    ? credential.name
                    : credential.name.substr(0, MAX_AUTOCOMPLETE_NAME_LEN) + '…';
         const group = (withGroup && credential.group) ? `[${credential.group}] ` : '';
         const visibleLogin = (credential.login.length > 0) ? credential.login : tr('credentialsNoUsername');
-        const text = `${group}${name} (${visibleLogin})`;
+        let text = `${group}${title} (${visibleLogin})`;
 
         if (credential.expired && credential.expired === 'true') {
-            return `${text} [${tr('credentialExpired')}]`;
+            text = `${text} [${tr('credentialExpired')}]`;
         }
 
-        return text;
+        return {
+            title: title,
+            group: group,
+            visibleLogin: visibleLogin,
+            login: credential.login,
+            loginId: loginId,
+            uuid: credential.uuid,
+            text: text
+        };
+    };
+
+    // Sorting with or without group name included
+    const sortLoginItemBy = function(a, b, name, withGroup = false) {
+        const firstGroup = a.group.toLowerCase();
+        const secondGroup = b.group.toLowerCase();
+        const first = a[name].toLowerCase();
+        const second = b[name].toLowerCase();
+
+        return withGroup
+            ? firstGroup.localeCompare(secondGroup) || first.localeCompare(second)
+            : first.localeCompare(second);
     };
 
     const getUniqueGroupCount = function(creds) {
@@ -1171,31 +1379,64 @@ kpxc.initLoginPopup = function() {
         return uniqueGroups.size;
     };
 
-    // Add usernames + descriptions to autocomplete-list and popup-list
-    const usernames = [];
-    kpxcAutocomplete.elements = [];
     const showGroupNameInAutocomplete = kpxc.settings.showGroupNameInAutocomplete && (getUniqueGroupCount(kpxc.credentials) > 1);
 
+    // Initialize login items
+    const loginItems = [];
     for (let i = 0; i < kpxc.credentials.length; i++) {
-        const loginText = getLoginText(kpxc.credentials[i], showGroupNameInAutocomplete);
-        usernames.push({ text: loginText, uuid: kpxc.credentials[i].uuid });
+        const loginItem = getLoginItem(kpxc.credentials[i], showGroupNameInAutocomplete, i);
+        loginItems.push(loginItem);
+    }
 
-        kpxcAutocomplete.elements.push({
-            label: loginText,
-            value: kpxc.credentials[i].login,
-            uuid: kpxc.credentials[i].uuid,
-            loginId: i
+    // Sort login items
+    if (kpxc.settings.credentialSorting === SORT_BY_TITLE) {
+        loginItems.sort((a, b) => sortLoginItemBy(a, b, 'title'));
+    } else if (kpxc.settings.credentialSorting === SORT_BY_USERNAME) {
+        loginItems.sort((a, b) => sortLoginItemBy(a, b, 'visibleLogin'));
+    } else if (kpxc.settings.credentialSorting === SORT_BY_GROUP_AND_TITLE) {
+        loginItems.sort((a, b) => sortLoginItemBy(a, b, 'title', true));
+    } else if (kpxc.settings.credentialSorting === SORT_BY_GROUP_AND_USERNAME) {
+        loginItems.sort((a, b) => sortLoginItemBy(a, b, 'visibleLogin', true));
+    }
+
+    const popupLoginItems = [];
+    kpxcUserAutocomplete.clear();
+
+    // Initialize Popup Login and Autocomplete Menu items
+    for (const l of loginItems) {
+        popupLoginItems.push({ text: l.text, uuid: l.uuid });
+
+        kpxcUserAutocomplete.elements.push({
+            label: l.text,
+            value: l.login,
+            uuid: l.uuid,
+            loginId: l.loginId
         });
     }
 
-    // Generate popup-list of usernames + descriptions
-    sendMessage('popup_login', usernames);
+    // Activate Popup Login list of usernames + descriptions
+    sendMessage('popup_login', popupLoginItems);
 };
 
 kpxc.passwordFilled = async function() {
     return await sendMessage('password_get_filled');
 };
 
+/**
+ * Handle passwordFilled() with possible exceptions, e.g. Protonmail's mailbox password
+ * where we actually need two passwords for a successful login.
+ * If an exception is found, act like password is not yet filled.
+ * @param {Object} currentForm  Current saved form. @see kpxcForm.saveForm
+ * @returns {boolean}           True if password has been already filled
+ */
+kpxc.passwordFilledWithExceptions = async function(currentForm) {
+    if (currentForm.password && kpxcSites.exceptionFound(currentForm.password.id)) {
+        return false;
+    }
+
+    return await sendMessage('password_get_filled');
+};
+
 // Prepares autocomplete and login popup ready for user interaction
 kpxc.prepareCredentials = async function() {
     if (kpxc.credentials.length === 0) {
@@ -1217,8 +1458,9 @@ kpxc.prepareCredentials = async function() {
  * @param {string} passwordValue    Submitted password
  * @param {string} urlValue         URL of the page where password change was detected
  * @param {Array} oldCredentials    Credentials saved from the password change page, if available
+ * @param {boolean} useBanner       If banner is disabled, save directly
  */
-kpxc.rememberCredentials = async function(usernameValue, passwordValue, urlValue, oldCredentials) {
+kpxc.rememberCredentials = async function(usernameValue, passwordValue, urlValue, oldCredentials, useBanner = true) {
     const credentials = (oldCredentials !== undefined && oldCredentials.length > 0) ? oldCredentials : kpxc.credentials;
     if (passwordValue === '') {
         return undefined;
@@ -1261,21 +1503,30 @@ kpxc.rememberCredentials = async function(usernameValue, passwordValue, urlValue
         if (credentialsList.length === 1) {
             usernameValue = credentialsList[0].login;
         } else if (credentialsList.length > 1) {
-            const index = await sendMessage('page_get_login_id');
-            if (index >= 0) {
-                usernameValue = credentialsList[index].login;
+            const uuid = await sendMessage('page_get_login_id');
+            if (uuid) {
+                const credsFromUuid = credentialsList.find(c => c.uuid === uuid);
+                if (credsFromUuid) {
+                    usernameValue = credsFromUuid.login;
+                }
             }
         }
     }
 
-    // Show the Credential Banner
-    kpxcBanner.create({
+    const saveCredentials = {
         username: usernameValue,
         password: passwordValue,
         url: urlValue,
         usernameExists: usernameExists,
         list: credentialsList
-    });
+    };
+
+    if (useBanner) {
+        kpxcBanner.create(saveCredentials);
+    } else {
+        kpxcBanner.credentials = saveCredentials;
+        kpxcBanner.saveNewCredentials(saveCredentials);
+    }
 
     return true;
 };
@@ -1289,10 +1540,14 @@ kpxc.rememberCredentialsFromContextMenu = async function() {
 
     const type = el.getAttribute('type');
     const combination = await kpxcFields.getCombination(el, (type === 'password' ? type : 'username'));
+    if (!combination) {
+        return;
+    }
+
     const usernameValue = combination.username ? combination.username.value : '';
     const passwordValue = combination.password ? combination.password.value : '';
 
-    const result = await kpxc.rememberCredentials(usernameValue, passwordValue);
+    const result = await kpxc.rememberCredentials(usernameValue, passwordValue, undefined, undefined, kpxc.settings.showLoginNotifications);
     if (result === undefined) {
         kpxcUI.createNotification('error', tr('rememberNoPassword'));
         return;
@@ -1303,13 +1558,14 @@ kpxc.rememberCredentialsFromContextMenu = async function() {
     }
 };
 
-// The basic function for retrieving credentials from KeePassXC
-kpxc.retrieveCredentials = async function() {
+// The basic function for retrieving credentials from KeePassXC.
+// Credential Banner can force the retrieval for reloading new/modified credentials.
+kpxc.retrieveCredentials = async function(force = false) {
     kpxc.url = document.location.href;
     kpxc.submitUrl = kpxc.getFormActionUrl(kpxc.combinations[0]);
 
     if (kpxc.settings.autoRetrieveCredentials && kpxc.url && kpxc.submitUrl) {
-        await kpxc.retrieveCredentialsCallback(await sendMessage('retrieve_credentials', [ kpxc.url, kpxc.submitUrl ]));
+        await kpxc.retrieveCredentialsCallback(await sendMessage('retrieve_credentials', [ kpxc.url, kpxc.submitUrl, force ]));
     }
 };
 
@@ -1383,8 +1639,8 @@ kpxc.setValue = function(field, value) {
 // Sets a new value to input field and triggers necessary events
 kpxc.setValueWithChange = function(field, value) {
     field.value = value;
-    field.dispatchEvent(new Event('input', { 'bubbles': true }));
-    field.dispatchEvent(new Event('change', { 'bubbles': true }));
+    field.dispatchEvent(new Event('input', { bubbles: true }));
+    field.dispatchEvent(new Event('change', { bubbles: true }));
     field.dispatchEvent(new KeyboardEvent('keydown', { bubbles: true, cancelable: false, key: '', char: '' }));
     field.dispatchEvent(new KeyboardEvent('keypress', { bubbles: true, cancelable: false, key: '', char: '' }));
     field.dispatchEvent(new KeyboardEvent('keyup', { bubbles: true, cancelable: false, key: '', char: '' }));
@@ -1395,11 +1651,11 @@ kpxc.siteIgnored = async function(condition) {
     if (kpxc.settings.sitePreferences) {
         let currentLocation;
         try {
-            currentLocation = window.top.location.href;
+            currentLocation = window.top.location.href.toLowerCase();
         } catch (err) {
             // Cross-domain security error inspecting window.top.location.href.
             // This catches an error when an iframe is being accessed from another (sub)domain -> use the iframe URL instead.
-            currentLocation = window.self.location.href;
+            currentLocation = window.self.location.href.toLowerCase();
         }
 
         const currentSetting = condition || IGNORE_FULL;
@@ -1450,6 +1706,36 @@ kpxc.updateDatabaseState = async function() {
     kpxc.databaseState = res.databaseClosed ? DatabaseState.LOCKED : DatabaseState.UNLOCKED;
 };
 
+// Updates the TOTP Autocomplete Menu
+kpxc.updateTOTPList = async function() {
+    let uuid = await sendMessage('page_get_login_id');
+    if (uuid === undefined || kpxc.credentials.length === 0) {
+        // Credential haven't been selected
+        return;
+    }
+
+    // Use the first credential available if not set
+    if (uuid === '') {
+        uuid = kpxc.credentials[0].uuid;
+    }
+
+    const credentials = kpxc.credentials.find(c => c.uuid === uuid);
+    if (credentials) {
+        const username = credentials.login;
+        const password = credentials.password;
+
+        // If no username is set, compare with a password
+        const credentialList = kpxc.credentials.filter(c => (c.totp || (c.stringFields && c.stringFields.some(s => s['KPH: {TOTP}'])))
+            && (c.login === username || (!username && c.password === password)));
+
+        // Filter TOTP Autocomplete Menu with matching 2FA credentials
+        kpxcTOTPAutocomplete.elements = kpxcUserAutocomplete.elements.filter(e => credentialList.some(u => u.uuid === e.uuid));
+        return credentialList;
+    }
+
+    return [];
+};
+
 
 /**
  * @Object kpxcObserverHelper
@@ -1542,7 +1828,7 @@ kpxcObserverHelper.getInputs = function(target, ignoreVisibility = false) {
     }
 
     // Append any input fields in Shadow DOM
-    if (target.shadowRoot) {
+    if (target.shadowRoot && typeof target.shadowSelectorAll === 'function') {
         target.shadowSelectorAll('input').forEach(e => {
             if (e.type !== 'hidden' && !e.disabled && !kpxcObserverHelper.alreadyIdentified(e)) {
                 inputFields.push(e);
@@ -1689,7 +1975,7 @@ kpxcObserverHelper.initObserver = async function() {
                 } else if (mut.removedNodes.length > 0) {
                     kpxcObserverHelper.handleObserverRemove(mut.removedNodes[0]);
                 }
-            } else if (mut.type === 'attributes' && mut.attributeName === 'class') {
+            } else if (mut.type === 'attributes' && (mut.attributeName === 'class' || mut.attributeName === 'style')) {
                 // Only accept targets with forms
                 const forms = mut.target.nodeName === 'FORM' ? mut.target : mut.target.getElementsByTagName('form');
                 if (forms.length === 0 && !kpxcSites.exceptionFound(mut.target.classList)) {
@@ -1813,10 +2099,16 @@ browser.runtime.onMessage.addListener(async function(req, sender) {
             kpxc.inputs = [];
             kpxc.combinations = [];
             kpxc.initCredentialFields();
-        } else if (req.action === 'remember_credentials') {
+        } else if (req.action === 'reload_extension') {
+            sendMessage('reconnect');
+        } else if (req.action === 'save_credentials') {
             kpxc.rememberCredentialsFromContextMenu();
+        } else if (req.action === 'retrive_credentials_forced') {
+            await kpxc.retrieveCredentials(true);
         } else if (req.action === 'show_password_generator') {
             kpxcPasswordDialog.trigger();
+        } else if (req.action === 'request_autotype') {
+            sendMessage('request_autotype', [ window.location.hostname ]);
         }
     }
 });
