@@ -70,23 +70,30 @@ PasswordIcon.prototype.createIcon = function(field) {
         icon.style.filter = 'saturate(0%)';
     }
 
-    icon.addEventListener('click', function(e) {
+    icon.addEventListener('click', async function(e) {
         if (!e.isTrusted) {
             return;
         }
 
-        e.preventDefault();
+        e.stopPropagation();
+
+        if (await useKeePassXCPasswordGenerator()) {
+            kpxcPasswordDialog.generate(null, field);
+            return;
+        }
+
         kpxcPasswordDialog.showDialog(field, icon);
     });
 
+    icon.addEventListener('mousedown', ev => ev.stopPropagation());
+    icon.addEventListener('mouseup', ev => ev.stopPropagation());
+
     kpxcUI.setIconPosition(icon, field, this.rtl);
     this.icon = icon;
 
-    const styleSheet = document.createElement('link');
-    styleSheet.setAttribute('rel', 'stylesheet');
-    styleSheet.setAttribute('href', browser.runtime.getURL('css/pwgen.css'));
-
+    const styleSheet = createStylesheet('css/pwgen.css');
     const wrapper = document.createElement('div');
+
     this.shadowRoot = wrapper.attachShadow({ mode: 'closed' });
     this.shadowRoot.append(styleSheet);
     this.shadowRoot.append(icon);
@@ -206,7 +213,12 @@ kpxcPasswordDialog.openDialog = function() {
     }
 };
 
-kpxcPasswordDialog.trigger = function() {
+kpxcPasswordDialog.trigger = async function() {
+    if (await useKeePassXCPasswordGenerator()) {
+        kpxcPasswordDialog.generate(null, document.activeElement);
+        return;
+    }
+
     kpxcPasswordDialog.showDialog(document.activeElement, kpxcPasswordDialog.icon);
 };
 
@@ -243,7 +255,7 @@ kpxcPasswordDialog.showDialog = function(field, icon) {
     }
 };
 
-kpxcPasswordDialog.generate = async function(e) {
+kpxcPasswordDialog.generate = async function(e, field) {
     // This function can be also called from non-events
     if (e) {
         if (!e.isTrusted) {
@@ -252,6 +264,11 @@ kpxcPasswordDialog.generate = async function(e) {
         e.preventDefault();
     }
 
+    if (await useKeePassXCPasswordGenerator()) {
+        kpxcPasswordDialog.newFill(field, await sendMessage('generate_password'));
+        return;
+    }
+
     callbackGeneratedPassword(await sendMessage('generate_password'));
 };
 
@@ -277,18 +294,52 @@ kpxcPasswordDialog.fill = function(e) {
             const message = tr('passwordGeneratorErrorTooLong') + '\r\n'
                             + tr('passwordGeneratorErrorTooLongCut') + '\r\n' + tr('passwordGeneratorErrorTooLongRemember');
             message.style.whiteSpace = 'pre';
-            sendMessage('show_notification', [ message ]);
+            kpxcUI.createNotification('error', message);
             return;
         }
     }
 
     kpxcPasswordDialog.input.value = password.value;
+    kpxcPasswordDialog.input.dispatchEvent(new Event('keydown', { bubbles: true }));
+    kpxcPasswordDialog.input.dispatchEvent(new Event('keyup', { bubbles: true }));
+    kpxcPasswordDialog.input.dispatchEvent(new Event('input', { bubbles: true }));
+    kpxcPasswordDialog.input.dispatchEvent(new Event('change', { bubbles: true }));
 
     if (kpxcPasswordDialog.nextField) {
         kpxcPasswordDialog.nextField.value = password.value;
+        kpxcPasswordDialog.nextField.dispatchEvent(new Event('keydown', { bubbles: true }));
+        kpxcPasswordDialog.nextField.dispatchEvent(new Event('keyup', { bubbles: true }));
+        kpxcPasswordDialog.nextField.dispatchEvent(new Event('input', { bubbles: true }));
+        kpxcPasswordDialog.nextField.dispatchEvent(new Event('change', { bubbles: true }));
     }
 };
 
+// New way to fill the password
+kpxcPasswordDialog.newFill = function(elem, password) {
+    if (!elem || !password) {
+        return;
+    }
+
+    if (password.length === 0) {
+        kpxcUI.createNotification('error', tr('usernameLockedFieldText'));
+        return;
+    }
+
+    if (elem.getAttribute('maxlength')) {
+        if (password.length > elem.getAttribute('maxlength')) {
+            const message = tr('passwordGeneratorErrorTooLong') + '\r\n'
+                            + tr('passwordGeneratorErrorTooLongCut') + '\r\n' + tr('passwordGeneratorErrorTooLongRemember');
+            message.style.whiteSpace = 'pre';
+            kpxcUI.createNotification('error', message);
+            return;
+        }
+    }
+
+    elem.value = password;
+    elem.dispatchEvent(new Event('input', { bubbles: true }));
+    elem.dispatchEvent(new Event('change', { bubbles: true }));
+};
+
 kpxcPasswordDialog.copyPasswordToClipboard = function() {
     kpxcPasswordDialog.shadowSelector('.kpxc-pwgen-input').select();
     try {
@@ -299,8 +350,8 @@ kpxcPasswordDialog.copyPasswordToClipboard = function() {
     return false;
 };
 
-const callbackGeneratedPassword = function(entries) {
-    if (entries && entries.length >= 1) {
+const callbackGeneratedPassword = function(passwords) {
+    if (passwords && passwords.length >= 1) {
         const errorMessage = kpxcPasswordDialog.shadowSelector('#kpxc-pwgen-error');
         if (errorMessage) {
             enableButtons();
@@ -310,7 +361,7 @@ const callbackGeneratedPassword = function(entries) {
             errorMessage.remove();
         }
 
-        kpxcPasswordDialog.shadowSelector('.kpxc-pwgen-input').value = entries[0].password;
+        kpxcPasswordDialog.shadowSelector('.kpxc-pwgen-input').value = passwords[0].password;
     } else {
         if (kpxcPasswordDialog.shadowSelectorAll('div#kpxc-pwgen-error').length === 0) {
             const input = kpxcPasswordDialog.shadowSelector('.kpxc-pwgen-input');
@@ -337,3 +388,16 @@ const disableButtons = function() {
     kpxcPasswordDialog.shadowSelector('#kpxc-pwgen-btn-copy').style.display = 'none';
     kpxcPasswordDialog.shadowSelector('#kpxc-pwgen-btn-fill').style.display = 'none';
 };
+
+const useKeePassXCPasswordGenerator = async function() {
+    const response = await browser.runtime.sendMessage({
+        action: 'get_keepassxc_versions'
+    });
+
+    const result = await browser.runtime.sendMessage({
+        action: 'compare_version',
+        args: [ '2.7.0', response.current ]
+    });
+
+    return result;
+};
