@@ -1,10 +1,11 @@
 'use strict';
 
-const ignoreRegex = /(bank|coupon|postal|user|zip).*code|comment|author/i;
+const ignoreRegex = /(bank|coupon|postal|user|zip).*code|comment|author|error/i;
 const ignoredTypes = [ 'email', 'password', 'username' ];
 
 const acceptedOTPFields = [
     '2fa',
+    '2fpin',
     'auth',
     'challenge',
     'code',
@@ -12,14 +13,21 @@ const acceptedOTPFields = [
     'mfa',
     'otp',
     'token',
-    'twofactor'
+    'twofa',
+    'two-factor',
+    'twofactor',
+    'verification_pin'
+];
+
+const acceptedParents = [
+    '.mfa-verify',
 ];
 
 var kpxcTOTPIcons = {};
 kpxcTOTPIcons.icons = [];
 
-kpxcTOTPIcons.newIcon = function(field, databaseState = DatabaseState.DISCONNECTED, forced = false) {
-    kpxcTOTPIcons.icons.push(new TOTPFieldIcon(field, databaseState, forced));
+kpxcTOTPIcons.newIcon = function(field, databaseState = DatabaseState.DISCONNECTED, segmented = false) {
+    kpxcTOTPIcons.icons.push(new TOTPFieldIcon(field, databaseState, segmented));
 };
 
 kpxcTOTPIcons.switchIcon = function(state) {
@@ -39,11 +47,15 @@ kpxcTOTPIcons.isAcceptedTOTPField = function(field) {
 
     // Checks if the field id, name or placeholder includes some of the acceptedOTPFields but not any from ignoredTypes
     if (autocomplete === 'one-time-code'
-        || (acceptedOTPFields.some(f => (id && id.includes(f)) || (name && name.includes(f) || placeholder && placeholder.includes(f))))
+        || (acceptedOTPFields.some(f => (id && id.includes(f)) || (name && name.includes(f) || placeholder && placeholder.includes(f))) || acceptedParents.some(s => field.closest(s)))
             && !ignoredTypes.some(f => (id && id.includes(f)) || (name && name.includes(f) || placeholder && placeholder.includes(f)))) {
         return true;
     }
 
+    if (kpxcSites.totpExceptionFound(field)) {
+        return true;
+    }
+
     return false;
 };
 
@@ -60,7 +72,9 @@ kpxcTOTPIcons.isValid = function(field, forced) {
             || ignoredTypes.some(t => t === field.autocomplete)
             || field.id.match(ignoreRegex)
             || field.name.match(ignoreRegex)
-            || field.readOnly) {
+            || field.placeholder.match(ignoreRegex)
+            || field.readOnly
+            || field.inputMode === 'email') {
             return false;
         }
     } else {
@@ -73,25 +87,25 @@ kpxcTOTPIcons.isValid = function(field, forced) {
 };
 
 class TOTPFieldIcon extends Icon {
-    constructor(field, databaseState = DatabaseState.DISCONNECTED, forced = false) {
-        super(field, databaseState);
+    constructor(field, databaseState = DatabaseState.DISCONNECTED, segmented = false) {
+        super(field, databaseState, segmented);
 
-        this.initField(field, forced);
+        this.initField(field, segmented);
         kpxcUI.monitorIconPosition(this);
     }
 }
 
-TOTPFieldIcon.prototype.initField = function(field, forced) {
+TOTPFieldIcon.prototype.initField = function(field, segmented) {
     // Observer the visibility
     if (this.observer) {
         this.observer.observe(field);
     }
 
-    this.createIcon(field);
+    this.createIcon(field, segmented);
     this.inputField = field;
 };
 
-TOTPFieldIcon.prototype.createIcon = function(field) {
+TOTPFieldIcon.prototype.createIcon = function(field, segmented = false) {
     const className = (isFirefox() ? 'moz' : 'default');
 
     // Size the icon dynamically, but not greater than 24 or smaller than 14
@@ -118,19 +132,20 @@ TOTPFieldIcon.prototype.createIcon = function(field) {
             return;
         }
 
-        e.preventDefault();
+        e.stopPropagation();
         await kpxc.receiveCredentialsIfNecessary();
         kpxc.fillFromTOTP(field);
     });
 
-    kpxcUI.setIconPosition(icon, field, this.rtl);
+    icon.addEventListener('mousedown', ev => ev.stopPropagation());
+    icon.addEventListener('mouseup', ev => ev.stopPropagation());
+
+    kpxcUI.setIconPosition(icon, field, this.rtl, segmented);
     this.icon = icon;
 
-    const styleSheet = document.createElement('link');
-    styleSheet.setAttribute('rel', 'stylesheet');
-    styleSheet.setAttribute('href', browser.runtime.getURL('css/totp.css'));
-
+    const styleSheet = createStylesheet('css/totp.css');
     const wrapper = document.createElement('div');
+
     this.shadowRoot = wrapper.attachShadow({ mode: 'closed' });
     this.shadowRoot.append(styleSheet);
     this.shadowRoot.append(icon);
