@@ -4,6 +4,11 @@ const MIN_TOTP_INPUT_LENGTH = 6;
 const MAX_TOTP_INPUT_LENGTH = 10;
 const MIN_INPUT_FIELD_WIDTH_PX = 8;
 const MIN_INPUT_FIELD_OFFSET_WIDTH = 60;
+const MIN_OPACITY = 0.7;
+const MAX_OPACITY = 1;
+
+let notificationWrapper;
+let notificationTimeout;
 
 const DatabaseState = {
     DISCONNECTED: 0,
@@ -23,11 +28,12 @@ const Pixels = function(value) {
 
 // Basic icon class
 class Icon {
-    constructor(field, databaseState = DatabaseState.DISCONNECTED) {
+    constructor(field, databaseState = DatabaseState.DISCONNECTED, segmented = false) {
         this.databaseState = databaseState;
         this.icon = null;
         this.inputField = null;
         this.rtl = kpxcUI.isRTL(field);
+        this.segmented = segmented;
 
         try {
             this.observer = new IntersectionObserver((entries) => {
@@ -103,7 +109,7 @@ kpxcUI.monitorIconPosition = function(iconClass) {
 
 kpxcUI.updateIconPosition = function(iconClass) {
     if (iconClass.inputField && iconClass.icon) {
-        kpxcUI.setIconPosition(iconClass.icon, iconClass.inputField, iconClass.rtl);
+        kpxcUI.setIconPosition(iconClass.icon, iconClass.inputField, iconClass.rtl, iconClass.segmented);
     }
 };
 
@@ -112,17 +118,24 @@ kpxcUI.calculateIconOffset = function(field, size) {
     return (offset < 0) ? 0 : offset;
 };
 
-kpxcUI.setIconPosition = function(icon, field, rtl = false) {
+kpxcUI.setIconPosition = function(icon, field, rtl = false, segmented = false) {
     const rect = field.getBoundingClientRect();
     const size = Number(icon.getAttribute('size'));
     const offset = kpxcUI.calculateIconOffset(field, size);
-    const left = kpxcUI.bodyStyle.position.toLowerCase() === 'relative' ? rect.left - kpxcUI.bodyRect.left : rect.left;
+    let left = kpxcUI.bodyStyle.position.toLowerCase() === 'relative' ? rect.left - kpxcUI.bodyRect.left : rect.left;
     const top = kpxcUI.bodyStyle.position.toLowerCase() === 'relative' ? rect.top - kpxcUI.bodyRect.top : rect.top;
 
-    icon.style.top = Pixels(top + document.scrollingElement.scrollTop + offset + 1);
+    // Add more space for the icon to show it at the right side of the field if TOTP fields are segmented
+    if (segmented) {
+        left += size + 10;
+    }
+
+    const scrollTop = document.scrollingElement ? document.scrollingElement.scrollTop : 0;
+    const scrollLeft = document.scrollingElement ? document.scrollingElement.scrollLeft : 0;
+    icon.style.top = Pixels(top + scrollTop + offset + 1);
     icon.style.left = rtl
-                    ? Pixels((left + document.scrollingElement.scrollLeft) + offset)
-                    : Pixels(left + document.scrollingElement.scrollLeft + field.offsetWidth - size - offset);
+                    ? Pixels((left + scrollLeft) + offset)
+                    : Pixels(left + scrollLeft + field.offsetWidth - size - offset);
 };
 
 kpxcUI.deleteHiddenIcons = function(iconList, attr) {
@@ -181,7 +194,7 @@ kpxcUI.updateFromIntersectionObserver = function(iconClass, entries) {
 
             // Wait for possible DOM animations
             setTimeout(() => {
-                kpxcUI.setIconPosition(iconClass.icon, entry.target, iconClass.rtl);
+                kpxcUI.setIconPosition(iconClass.icon, entry.target, iconClass.rtl, iconClass.segmented);
             }, 400);
         }
     }
@@ -197,7 +210,7 @@ kpxcUI.createNotification = function(type, message) {
         return;
     }
 
-    const banner = kpxcUI.createElement('div', 'kpxc-notification kpxc-notification-' + type, {});
+    const notification = kpxcUI.createElement('div', 'kpxc-notification kpxc-notification-' + type, {});
     type = type.charAt(0).toUpperCase() + type.slice(1) + '!';
 
     const className = (isFirefox() ? 'kpxc-banner-icon-moz' : 'kpxc-banner-icon');
@@ -205,17 +218,35 @@ kpxcUI.createNotification = function(type, message) {
     const label = kpxcUI.createElement('span', 'kpxc-label', {}, type);
     const msg = kpxcUI.createElement('span', '', {}, message);
 
-    banner.addEventListener('click', function() {
-        document.body.removeChild(banner);
+    notification.addEventListener('click', function() {
+        if (notificationWrapper && window.parent.document.body.contains(notificationWrapper)) {
+            window.parent.document.body.removeChild(notificationWrapper);
+            notificationWrapper = undefined;
+        }
     });
 
-    banner.appendMultiple(icon, label, msg);
-    document.body.appendChild(banner);
+    notification.appendMultiple(icon, label, msg);
+
+    const styleSheet = createStylesheet('css/notification.css');
+    notificationWrapper = notificationWrapper || document.createElement('div');
+    this.shadowRoot = notificationWrapper.attachShadow({ mode: 'closed' });
+    if (!this.shadowRoot) {
+        return;
+    }
+
+    this.shadowRoot.append(styleSheet);
+    this.shadowRoot.append(notification);
+    document.body.append(notificationWrapper);
+
+    if (notificationTimeout) {
+        clearTimeout(notificationTimeout);
+    }
 
     // Destroy the banner after five seconds
-    setTimeout(() => {
-        if ($('.kpxc-notification')) {
-            document.body.removeChild(banner);
+    notificationTimeout = setTimeout(() => {
+        if (notificationWrapper && window.parent.document.body.contains(notificationWrapper)) {
+            window.parent.document.body.removeChild(notificationWrapper);
+            notificationWrapper = undefined;
         }
     }, 5000);
 };
@@ -261,18 +292,26 @@ document.addEventListener('mousemove', function(e) {
         const xPos = e.clientX - kpxcDefine.diffX;
         const yPos = e.clientY - kpxcDefine.diffY;
 
-        if (kpxcDefine.selected !== null) {
+        if (kpxcDefine.selected && kpxcDefine.dialog) {
             kpxcDefine.dialog.style.left = Pixels(xPos);
             kpxcDefine.dialog.style.top = Pixels(yPos);
         }
     }
 });
 
-document.addEventListener('mousedown', function() {
+document.addEventListener('mousedown', function(e) {
+    if (!e.isTrusted) {
+        return;
+    }
+
     kpxcUI.mouseDown = true;
 });
 
-document.addEventListener('mouseup', function() {
+document.addEventListener('mouseup', function(e) {
+    if (!e.isTrusted) {
+        return;
+    }
+
     kpxcPasswordDialog.selected = null;
     kpxcDefine.selected = null;
     kpxcUI.mouseDown = false;
@@ -290,7 +329,11 @@ Element.prototype.getLowerCaseAttribute = function(attr) {
 
 Element.prototype._attachShadow = Element.prototype.attachShadow;
 Element.prototype.attachShadow = function () {
-    return this._attachShadow({ mode: 'closed' });
+    try {
+        return this._attachShadow({ mode: 'closed' });
+    } catch (e) {
+        console.log('Error: ', e);
+    }
 };
 
 Object.prototype.shadowSelector = function(value) {
