@@ -73,12 +73,19 @@ options.initGeneralSettings = function() {
         $('#tab-general-settings select#colorTheme').val(options.settings['colorTheme']);
     }
 
-    $('#tab-general-settings select:first').change(async function() {
+    $('#tab-general-settings select#colorTheme').change(async function() {
         options.settings['colorTheme'] = $(this).val();
+        // The theme is also stored in localStorage to prevent a white flash when the settings are first opened
+        localStorage.setItem('colorTheme', options.settings['colorTheme']);
         await options.saveSettings();
         location.reload();
     });
 
+    $('#tab-general-settings select#credentialSorting').change(async function() {
+        options.settings['credentialSorting'] = $(this).val();
+        await options.saveSettings();
+    });
+
     $('#tab-general-settings input[type=checkbox]').each(function() {
         $(this).attr('checked', options.settings[$(this).attr('name')]);
         if ($(this).attr('name') === 'defaultGroupAlwaysAsk' && $(this).attr('checked')) {
@@ -88,6 +95,11 @@ options.initGeneralSettings = function() {
         }
     });
 
+    $('#tab-general-settings input[type=radio]#checkUpdateThreeDays').val(CHECK_UPDATE_THREE_DAYS);
+    $('#tab-general-settings input[type=radio]#checkUpdateOneWeek').val(CHECK_UPDATE_ONE_WEEK);
+    $('#tab-general-settings input[type=radio]#checkUpdateOneMonth').val(CHECK_UPDATE_ONE_MONTH);
+    $('#tab-general-settings input[type=radio]#checkUpdateNever').val(CHECK_UPDATE_NEVER);
+
     $('#tab-general-settings input[type=range]').val(options.settings['redirectAllowance']);
     $('#redirectAllowanceLabel').text(tr('optionsRedirectAllowance',
         options.settings['redirectAllowance'] === 11 ? 'Infinite' : String(options.settings['redirectAllowance'])));
@@ -115,8 +127,19 @@ options.initGeneralSettings = function() {
         }
     });
 
+    $('#tab-general-settings select#credentialSorting').val(options.settings['credentialSorting']);
     $('#tab-general-settings input#defaultGroup').val(options.settings['defaultGroup']);
 
+    $('#tab-general-settings input#clearCredentialTimeout').val(options.settings['clearCredentialsTimeout']);
+    $('#tab-general-settings input#clearCredentialTimeout').change(async function(e) {
+        if (e.target.valueAsNumber < 0 || e.target.valueAsNumber > 3600) {
+            return;
+        }
+
+        options.settings['clearCredentialsTimeout'] = e.target.valueAsNumber;
+        await options.saveSettings();
+    });
+
     $('#tab-general-settings input[type=radio]').each(function() {
         if ($(this).val() === String(options.settings[$(this).attr('name')])) {
             $(this).attr('checked', options.settings[$(this).attr('name')]);
@@ -257,7 +280,8 @@ options.initGeneralSettings = function() {
     }
 };
 
-options.showKeePassXCVersions = function(response) {
+// Also hides/disables any options with KeePassXC versions that are too old
+options.showKeePassXCVersions = async function(response) {
     if (response.current === '') {
         response.current = 'unknown';
     }
@@ -270,11 +294,27 @@ options.showKeePassXCVersions = function(response) {
     $('#tab-about span.kpxcVersion').text(response.current);
     $('#tab-general-settings button.checkUpdateKeePassXC:first').attr('disabled', false);
 
-    if (response.current.startsWith('2.6') || response.current === '2.5.3-snapshot') {
+    // Hide/disable certain options with older KeePassXC versions than 2.6.0
+    const version260Result = await browser.runtime.sendMessage({
+        action: 'compare_version',
+        args: [ '2.6.0', response.current ]
+    });
+
+    if (version260Result) {
         $('#tab-general-settings #versionRequiredAlert').hide();
     } else {
         $('#tab-general-settings #showGroupNameInAutocomplete').attr('disabled', true);
     }
+
+    // Hide certain options with older KeePassXC versions than 2.7.0
+    const version270Result = await browser.runtime.sendMessage({
+        action: 'compare_version',
+        args: [ '2.7.0', response.current ]
+    });
+
+    if (!version270Result) {
+        $('#tab-general-settings #downloadFaviconAfterSaveFormGroup').hide();
+    }
 };
 
 options.getPartiallyHiddenKey = function(key) {
@@ -317,24 +357,24 @@ options.initConnectedDatabases = function() {
 
     $('#tab-connected-databases tr.clone:first .dropdown-menu:first').width('230px');
 
-    const trClone = $('#tab-connected-databases table tr.clone:first').clone(true);
-    trClone.removeClass('clone d-none');
+    const rowClone = $('#tab-connected-databases table tr.clone:first').clone(true);
+    rowClone.removeClass('clone d-none');
 
     const addHashToTable = function(hash) {
         $('#tab-connected-databases table tbody:first tr.empty:first').hide();
-        const tr = trClone.clone(true);
-        tr.data('hash', hash);
-        tr.attr('id', 'tr-cd-' + hash);
+        const row = rowClone.clone(true);
+        row.data('hash', hash);
+        row.attr('id', 'tr-cd-' + hash);
 
-        $('a.dropdown-toggle:first img:first', tr).attr('src', '/icons/toolbar/icon_normal.png');
+        $('a.dropdown-toggle:first img:first', row).attr('src', '/icons/toolbar/icon_normal.png');
 
-        tr.children('td:first').text(options.keyRing[hash].id);
-        tr.children('td:eq(1)').text(options.getPartiallyHiddenKey(options.keyRing[hash].key));
+        row.children('td:first').text(options.keyRing[hash].id);
+        row.children('td:eq(1)').text(options.getPartiallyHiddenKey(options.keyRing[hash].key));
         const lastUsed = (options.keyRing[hash].lastUsed) ? new Date(options.keyRing[hash].lastUsed).toLocaleString() : 'unknown';
-        tr.children('td:eq(2)').text(lastUsed);
+        row.children('td:eq(2)').text(lastUsed);
         const date = (options.keyRing[hash].created) ? new Date(options.keyRing[hash].created).toLocaleDateString() : 'unknown';
-        tr.children('td:eq(3)').text(date);
-        $('#tab-connected-databases table tbody:first').append(tr);
+        row.children('td:eq(3)').text(date);
+        $('#tab-connected-databases table tbody:first').append(row);
     };
 
     let hashList = options.keyRing;
@@ -400,17 +440,17 @@ options.initCustomCredentialFields = function() {
         }
     });
 
-    const trClone = $('#tab-custom-fields table tr.clone:first').clone(true);
-    trClone.removeClass('clone d-none');
+    const rowClone = $('#tab-custom-fields table tr.clone:first').clone(true);
+    rowClone.removeClass('clone d-none');
     let counter = 1;
     for (const url in options.settings['defined-custom-fields']) {
-        const tr = trClone.clone(true);
-        tr.data('url', url);
-        tr.attr('id', 'tr-scf' + counter);
+        const row = rowClone.clone(true);
+        row.data('url', url);
+        row.attr('id', 'tr-scf' + counter);
         ++counter;
 
-        tr.children('td:first').text(url);
-        $('#tab-custom-fields table tbody:first').append(tr);
+        row.children('td:first').text(url);
+        $('#tab-custom-fields table tbody:first').append(row);
     }
 
     if ($('#tab-custom-fields table tbody:first tr').length > 2) {
@@ -479,40 +519,38 @@ options.initSitePreferences = function() {
             return;
         }
 
-        const errorMessage = tr('optionsErrorValueExists');
-        let value = manualUrl.value;
-        if (value.length > 10 && value.length <= 2000) {
-            // Fills the last / char if needed. This ensures the compatibility with Match Patterns
-            if (slashNeededForUrl(value)) {
-                value += '/';
-            }
+        let value = manualUrl.value.toLowerCase();
 
-            // Check if the URL is already in the list
-            if (options.settings['sitePreferences'].some(s => s.url === value)) {
-                options.createWarning(manualUrl, errorMessage);
-                return;
-            }
-
-            if (options.settings['sitePreferences'] === undefined) {
-                options.settings['sitePreferences'] = [];
-            }
-
-            const newValue = options.settings['sitePreferences'].length + 1;
-            const trClone = $('#tab-site-preferences table tr.clone:first').clone(true);
-            trClone.removeClass('clone d-none');
-
-            const tr = trClone.clone(true);
-            tr.data('url', value);
-            tr.attr('id', 'tr-scf' + newValue);
-            tr.children('td:first').text(value);
-            tr.children('td:nth-child(2)').children('select').val(IGNORE_NOTHING);
-            $('#tab-site-preferences table tbody:first').append(tr);
-            $('#tab-site-preferences table tbody:first tr.empty:first').hide();
-
-            options.settings['sitePreferences'].push({ url: value, ignore: IGNORE_NOTHING, usernameOnly: false });
-            options.saveSettings();
-            manualUrl.value = '';
+        // Fills the last / char if needed. This ensures the compatibility with Match Patterns
+        if (slashNeededForUrl(value)) {
+            value += '/';
         }
+
+        // Check if the URL is already in the list
+        if (options.settings['sitePreferences'].some(s => s.url === value)) {
+            options.createWarning(manualUrl, tr('optionsErrorValueExists'));
+            return;
+        }
+
+        if (options.settings['sitePreferences'] === undefined) {
+            options.settings['sitePreferences'] = [];
+        }
+
+        const newValue = options.settings['sitePreferences'].length + 1;
+        const rowClone = $('#tab-site-preferences table tr.clone:first').clone(true);
+        rowClone.removeClass('clone d-none');
+
+        const row = rowClone.clone(true);
+        row.data('url', value);
+        row.attr('id', 'tr-scf' + newValue);
+        row.children('td:first').text(value);
+        row.children('td:nth-child(2)').children('select').val(IGNORE_NOTHING);
+        $('#tab-site-preferences table tbody:first').append(row);
+        $('#tab-site-preferences table tbody:first tr.empty:first').hide();
+
+        options.settings['sitePreferences'].push({ url: value, ignore: IGNORE_NOTHING, usernameOnly: false });
+        options.saveSettings();
+        manualUrl.value = '';
     });
 
     $('#dialogDeleteSite .modal-footer:first button.yes:first').click(function(e) {
@@ -536,20 +574,20 @@ options.initSitePreferences = function() {
         }
     });
 
-    const trClone = $('#tab-site-preferences table tr.clone:first').clone(true);
-    trClone.removeClass('clone d-none');
+    const rowClone = $('#tab-site-preferences table tr.clone:first').clone(true);
+    rowClone.removeClass('clone d-none');
     let counter = 1;
     if (options.settings['sitePreferences']) {
         for (const site of options.settings['sitePreferences']) {
-            const tr = trClone.clone(true);
-            tr.data('url', site.url);
-            tr.attr('id', 'tr-scf' + counter);
+            const row = rowClone.clone(true);
+            row.data('url', site.url);
+            row.attr('id', 'tr-scf' + counter);
             ++counter;
 
-            tr.children('td:first').text(site.url);
-            tr.children('td:nth-child(2)').children('select').val(site.ignore);
-            tr.children('td:nth-child(3)').children('input[type=checkbox]').attr('checked', site.usernameOnly);
-            $('#tab-site-preferences table tbody:first').append(tr);
+            row.children('td:first').text(site.url);
+            row.children('td:nth-child(2)').children('select').val(site.ignore);
+            row.children('td:nth-child(3)').children('input[type=checkbox]').attr('checked', site.usernameOnly);
+            $('#tab-site-preferences table tbody:first').append(row);
         }
     }
 
@@ -562,9 +600,14 @@ options.initSitePreferences = function() {
 
 options.initAbout = function() {
     const version = browser.runtime.getManifest().version;
+    let platform = navigator.platform;
+    if (platform === 'Win32' && (navigator.userAgent.includes('x64') || navigator.userAgent.includes('WOW64'))) {
+        platform = 'Win64';
+    }
+
     $('#tab-about em.versionCIP').text(version);
     $('#tab-about span.kpxcbrVersion').text(version);
-    $('#tab-about span.kpxcbrOS').text(navigator.platform);
+    $('#tab-about span.kpxcbrOS').text(platform);
     $('#tab-about span.kpxcbrBrowser').text(getBrowserId());
 
     // Hides keyboard shortcut configure button if Firefox version is < 60 (API is not compatible)
@@ -579,6 +622,11 @@ options.initTheme = function() {
     } else {
         document.body.setAttribute('data-color-theme', options.settings['colorTheme']);
     }
+    // Sync localStorage setting
+    const localStorageTheme = localStorage.getItem('colorTheme');
+    if (localStorageTheme !== options.settings['colorTheme']) {
+        localStorage.setItem('colorTheme', options.settings['colorTheme']);
+    }
 };
 
 options.createWarning = function(elem, text) {
