@@ -57,6 +57,9 @@ $(async () => {
         await browser.runtime.sendMessage({
             action: 'associate'
         });
+
+        // This does not work with Firefox because of https://bugzilla.mozilla.org/show_bug.cgi?id=1665380
+        await sendMessageToTab('retrive_credentials_forced');
         close();
     });
 
